@@ -31,19 +31,20 @@
 #include <QLocale>
 #include <QTranslator>
 #include <QFile>
+#include <QLibraryInfo>
 
 #ifndef DISABLE_GUI
 #include <QMessageBox>
 #include <QStyleFactory>
 #include <QStyle>
 #include <QSplashScreen>
+#include <QPainter>
+#include <QPen>
+#include <QFont>
 #include <QPushButton>
-#ifdef Q_WS_MAC
-#include "qmacapplication.h"
-#else
-#include "qtsingleapplication.h"
-#endif
-#include "GUI.h"
+#include <QTimer>
+#include "sessionapplication.h"
+#include "mainwindow.h"
 #include "ico.h"
 #else
 #include "qtsinglecoreapplication.h"
@@ -52,6 +53,7 @@
 #include "headlessloader.h"
 #endif
 
+#include "preferences.h"
 #include "qinisettings.h"
 #if defined(Q_WS_X11) || defined(Q_WS_MAC)
 #include <signal.h>
@@ -74,7 +76,7 @@ public:
     std::cout << '\t' << prg_name << " --no-splash: " << qPrintable(tr("disable splash screen")) << std::endl;
 #endif
     std::cout << '\t' << prg_name << " --help: " << qPrintable(tr("displays this help message")) << std::endl;
-    std::cout << '\t' << prg_name << " --webui-port=x: " << qPrintable(tr("changes the webui port (current: %1)").arg(QString::number(Preferences::getWebUiPort()))) << std::endl;
+    std::cout << '\t' << prg_name << " --webui-port=x: " << qPrintable(tr("changes the webui port (current: %1)").arg(QString::number(Preferences().getWebUiPort()))) << std::endl;
     std::cout << '\t' << prg_name << " " << qPrintable(tr("[files or urls]: downloads the torrents passed by the user (optional)")) << std::endl;
   }
 };
@@ -104,6 +106,8 @@ public:
     msgBox.setWindowTitle(tr("Legal notice"));
     msgBox.addButton(tr("Cancel"), QMessageBox::RejectRole);
     QAbstractButton *agree_button = msgBox.addButton(tr("I Agree"), QMessageBox::AcceptRole);
+    msgBox.show(); // Need to be shown or to moveToCenter does not work
+    msgBox.move(misc::screenCenter(&msgBox));
     msgBox.exec();
     if(msgBox.clickedButton() == agree_button) {
       // Save the answer
@@ -134,6 +138,7 @@ void sigsegvHandler(int) {
   signal(SIGSEGV, 0);
   std::cerr << "\n\n*************************************************************\n";
   std::cerr << "Catching SIGSEGV, please report a bug at http://bug.qbittorrent.org\nand provide the following backtrace:\n";
+  std::cerr << "qBittorrent version: " << VERSION << std::endl;
   print_stacktrace();
   raise(SIGSEGV);
 }
@@ -142,6 +147,7 @@ void sigabrtHandler(int) {
   signal(SIGSEGV, 0);
   std::cerr << "\n\n*************************************************************\n";
   std::cerr << "Catching SIGABRT, please report a bug at http://bug.qbittorrent.org\nand provide the following backtrace:\n";
+  std::cerr << "qBittorrent version: " << VERSION << std::endl;
   print_stacktrace();
   raise(SIGABRT);
 }
@@ -152,7 +158,7 @@ void useStyle(QString style){
   if(!style.isEmpty()) {
     QApplication::setStyle(QStyleFactory::create(style));
   }
-  Preferences::setStyle(QApplication::style()->objectName());
+  Preferences().setStyle(QApplication::style()->objectName());
 }
 #endif
 
@@ -160,15 +166,11 @@ void useStyle(QString style){
 int main(int argc, char *argv[]){
   // Create Application
   QString uid = misc::getUserIDString();
-  #ifdef DISABLE_GUI
+#ifdef DISABLE_GUI
   QtSingleCoreApplication app("qBittorrent-"+uid, argc, argv);
-  #else
-  #ifndef Q_WS_MAC
-  QtSingleApplication app("qBittorrent-"+uid, argc, argv);
-  #else
-  QMacApplication app("qBittorrent-"+uid, argc, argv);
-  #endif
-  #endif
+#else
+  SessionApplication app("qBittorrent-"+uid, argc, argv);
+#endif
 
   // Check if qBittorrent is already running for this user
   if(app.isRunning()) {
@@ -190,19 +192,27 @@ int main(int argc, char *argv[]){
     return 0;
   }
 
-  QString locale;
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  Preferences pref;
 #ifndef DISABLE_GUI
   bool no_splash = false;
 #endif
 
   // Load translation
-  locale = settings.value(QString::fromUtf8("Preferences/General/Locale"), QString()).toString();
+  QString locale = pref.getLocale();
+  QTranslator qtTranslator;
   QTranslator translator;
   if(locale.isEmpty()){
     locale = QLocale::system().name();
-    settings.setValue(QString::fromUtf8("Preferences/General/Locale"), locale);
+    pref.setLocale(locale);
   }
+  if(qtTranslator.load(
+          QString::fromUtf8("qt_") + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath)
+                                                                    )){
+    qDebug("Qt %s locale recognized, using translation.", qPrintable(locale));
+  }else{
+    qDebug("Qt %s locale unrecognized, using default (en_GB).", qPrintable(locale));
+  }
+  app.installTranslator(&qtTranslator);
   if(translator.load(QString::fromUtf8(":/lang/qbittorrent_") + locale)){
     qDebug("%s locale recognized, using translation.", qPrintable(locale));
   }else{
@@ -242,7 +252,7 @@ int main(int argc, char *argv[]){
             bool ok = false;
             int new_port = parts.last().toInt(&ok);
             if(ok && new_port > 0 && new_port <= 65535) {
-              Preferences::setWebUiPort(new_port);
+              Preferences().setWebUiPort(new_port);
             }
           }
         }
@@ -253,9 +263,22 @@ int main(int argc, char *argv[]){
   }
 
 #ifndef DISABLE_GUI
-  if(settings.value(QString::fromUtf8("Preferences/General/NoSplashScreen"), false).toBool()) {
+  if(pref.isSlashScreenDisabled()) {
     no_splash = true;
   }
+  QSplashScreen *splash = 0;
+  if(!no_splash) {
+    QPixmap splash_img(":/Icons/skin/splash.png");
+    QPainter painter(&splash_img);
+    QString version = VERSION;
+    painter.setPen(QPen(Qt::white));
+    painter.setFont(QFont("Arial", 22, QFont::Black));
+    painter.drawText(224 - painter.fontMetrics().width(version), 270, version);
+    splash = new QSplashScreen(splash_img, Qt::WindowStaysOnTopHint);
+    QTimer::singleShot(1500, splash, SLOT(deleteLater()));
+    splash->show();
+    app.processEvents();
+  }
 #endif
   // Set environment variable
   if(putenv((char*)"QBITTORRENT="VERSION)) {
@@ -263,19 +286,11 @@ int main(int argc, char *argv[]){
   }
 
 #ifndef DISABLE_GUI
-  useStyle(settings.value("Preferences/General/Style", "").toString());
+  useStyle(pref.getStyle());
   app.setStyleSheet("QStatusBar::item { border-width: 0; }");
-  QSplashScreen *splash = 0;
-  if(!no_splash) {
-    splash = new QSplashScreen(QPixmap(QString::fromUtf8(":/Icons/skin/splash.png")));
-    splash->show();
-  }
 #endif
 
   if(!LegalNotice::userAgreesWithNotice()) {
-#ifndef DISABLE_GUI
-    delete splash;
-#endif
     return 0;
   }
 #ifndef DISABLE_GUI
@@ -291,12 +306,14 @@ int main(int argc, char *argv[]){
   QStringList torrentCmdLine = app.arguments();
   // Remove first argument (program name)
   torrentCmdLine.removeFirst();
-#ifndef DISABLE_GUI
-  GUI window(0, torrentCmdLine);
-  if(!no_splash) {
-    splash->finish(&window);
-    delete splash;
+#ifndef QT_NO_DEBUG_OUTPUT
+  foreach(const QString &argument, torrentCmdLine) {
+    qDebug() << "Command line argument:" << argument;
   }
+#endif
+
+#ifndef DISABLE_GUI
+  MainWindow window(0, torrentCmdLine);
   QObject::connect(&app, SIGNAL(messageReceived(const QString&)),
                    &window, SLOT(processParams(const QString&)));
   app.setActivationWindow(&window);
