@@ -35,6 +35,7 @@
 #include <QFileInfo>
 #include <QDateTime>
 #include <QByteArray>
+#include <QDebug>
 
 #ifdef DISABLE_GUI
 #include <QCoreApplication>
@@ -46,6 +47,7 @@
 #ifdef Q_WS_WIN
 #include <shlobj.h>
 #include <windows.h>
+#include <PowrProf.h>
 const int UNLEN = 256;
 #else
 #include <unistd.h>
@@ -58,7 +60,7 @@ const int UNLEN = 256;
 #endif
 
 #ifndef Q_WS_WIN
-#ifdef Q_WS_MAC
+#if defined(Q_WS_MAC) || defined(Q_OS_FREEBSD)
 #include <sys/param.h>
 #include <sys/mount.h>
 #else
@@ -68,10 +70,22 @@ const int UNLEN = 256;
 #include <winbase.h>
 #endif
 
+#ifndef DISABLE_GUI
 #ifdef Q_WS_X11
 #include <QDBusInterface>
 #include <QDBusMessage>
 #endif
+#endif // DISABLE_GUI
+
+using namespace libtorrent;
+
+static struct { const char *source; const char *comment; } units[] = {
+    QT_TRANSLATE_NOOP3("misc", "B", "bytes"),
+    QT_TRANSLATE_NOOP3("misc", "KiB", "kibibytes (1024 bytes)"),
+    QT_TRANSLATE_NOOP3("misc", "MiB", "mebibytes (1024 kibibytes)"),
+    QT_TRANSLATE_NOOP3("misc", "GiB", "gibibytes (1024 mibibytes)"),
+    QT_TRANSLATE_NOOP3("misc", "TiB", "tebibytes (1024 gibibytes)")
+    };
 
 QString misc::QDesktopServicesDataLocation() {
 #ifdef Q_WS_WIN
@@ -80,9 +94,9 @@ QString misc::QDesktopServicesDataLocation() {
 #if defined Q_WS_WINCE
   if (SHGetSpecialFolderPath(0, path, CSIDL_APPDATA, FALSE))
 #else
-    if (SHGetSpecialFolderPath(0, path, CSIDL_LOCAL_APPDATA, FALSE))
+  if (SHGetSpecialFolderPath(0, path, CSIDL_LOCAL_APPDATA, FALSE))
 #endif
-      result = QString::fromWCharArray(path);
+    result = QString::fromWCharArray(path);
   if (!QCoreApplication::applicationName().isEmpty())
     result = result + QLatin1String("\\") + qApp->applicationName();
   if(!result.endsWith("\\"))
@@ -106,7 +120,7 @@ QString misc::QDesktopServicesDataLocation() {
   if (xdgDataHome.isEmpty())
     xdgDataHome = QDir::homePath() + QLatin1String("/.local/share");
   xdgDataHome += QLatin1String("/data/")
-                 + qApp->applicationName();
+      + qApp->applicationName();
   return xdgDataHome;
 #endif
 #endif
@@ -158,7 +172,7 @@ long long misc::freeDiskSpaceOnPath(QString path) {
   const int ret = statfs (qPrintable(statfs_path), &stats) ;
   if(ret == 0) {
     available = ((unsigned long long)stats.f_bavail) *
-                ((unsigned long long)stats.f_bsize) ;
+        ((unsigned long long)stats.f_bsize) ;
     return available;
   } else {
     return -1;
@@ -170,10 +184,10 @@ long long misc::freeDiskSpaceOnPath(QString path) {
                                               PULARGE_INTEGER);
   GetDiskFreeSpaceEx_t
       pGetDiskFreeSpaceEx = (GetDiskFreeSpaceEx_t)::GetProcAddress
-                            (
-                                ::GetModuleHandle(TEXT("kernel32.dll")),
-                                "GetDiskFreeSpaceExW"
-                                );
+      (
+        ::GetModuleHandle(TEXT("kernel32.dll")),
+        "GetDiskFreeSpaceExW"
+        );
   if ( pGetDiskFreeSpaceEx )
   {
     ULARGE_INTEGER bytesFree, bytesTotal;
@@ -190,15 +204,23 @@ long long misc::freeDiskSpaceOnPath(QString path) {
 #endif
 }
 
-void misc::shutdownComputer() {
+#ifndef DISABLE_GUI
+void misc::shutdownComputer(bool sleep) {
 #ifdef Q_WS_X11
   // Use dbus to power off the system
   // dbus-send --print-reply --system --dest=org.freedesktop.Hal /org/freedesktop/Hal/devices/computer org.freedesktop.Hal.Device.SystemPowerManagement.Shutdown
   QDBusInterface computer("org.freedesktop.Hal", "/org/freedesktop/Hal/devices/computer", "org.freedesktop.Hal.Device.SystemPowerManagement", QDBusConnection::systemBus());
-  computer.call("Shutdown");
+  if(sleep)
+    computer.call("Suspend", 5);
+  else
+    computer.call("Shutdown");
 #endif
 #ifdef Q_WS_MAC
-  AEEventID EventToSend = kAEShutDown;
+  AEEventID EventToSend;
+  if(sleep)
+    EventToSend = kAESleep;
+  else
+    EventToSend = kAEShutDown;
   AEAddressDesc targetDesc;
   static const ProcessSerialNumber kPSNOfSystemProcess = { 0, kSystemProcess };
   AppleEvent eventReply = {typeNull, NULL};
@@ -220,7 +242,7 @@ void misc::shutdownComputer() {
   AEDisposeDesc(&targetDesc);
   if (error != noErr)
   {
-      return;
+    return;
   }
 
   error = AESend(&appleEventToSend, &eventReply, kAENoReply,
@@ -255,8 +277,11 @@ void misc::shutdownComputer() {
 
   if (GetLastError() != ERROR_SUCCESS)
     return;
-  bool ret = InitiateSystemShutdownA(0, tr("qBittorrent will shutdown the computer now because all downloads are complete.").toLocal8Bit().data(), 10, true, false);
-  qDebug("ret: %d", (int)ret);
+  bool ret;
+  if(sleep)
+    SetSuspendState(false, false, false);
+  else
+    InitiateSystemShutdownA(0, tr("qBittorrent will shutdown the computer now because all downloads are complete.").toLocal8Bit().data(), 10, true, false);
 
   // Disable shutdown privilege.
   tkp.Privileges[0].Attributes = 0;
@@ -264,20 +289,31 @@ void misc::shutdownComputer() {
                         (PTOKEN_PRIVILEGES) NULL, 0);
 #endif
 }
+#endif // DISABLE_GUI
 
 QString misc::truncateRootFolder(boost::intrusive_ptr<torrent_info> t) {
+#if LIBTORRENT_VERSION_MINOR >= 16
+  file_storage fs = t->files();
+#endif
   if(t->num_files() == 1) {
     // Single file torrent
     // Remove possible subfolders
+#if LIBTORRENT_VERSION_MINOR >= 16
+    QString path = toQStringU(fs.file_path(t->file_at(0)));
+#else
     QString path = QString::fromUtf8(t->file_at(0).path.string().c_str());
-    QStringList path_parts = path.split("/", QString::SkipEmptyParts);
-    t->rename_file(0, path_parts.last().toUtf8().data());
+#endif
+    t->rename_file(0, fileName(path).toUtf8().data());
     return QString::null;
   }
   QString root_folder;
   int i = 0;
   for(torrent_info::file_iterator it = t->begin_files(); it < t->end_files(); it++) {
+#if LIBTORRENT_VERSION_MINOR >= 16
+    QString path = toQStringU(fs.file_path(*it));
+#else
     QString path = QString::fromUtf8(it->path.string().c_str());
+#endif
     QStringList path_parts = path.split("/", QString::SkipEmptyParts);
     if(path_parts.size() > 1) {
       root_folder = path_parts.takeFirst();
@@ -290,18 +326,28 @@ QString misc::truncateRootFolder(boost::intrusive_ptr<torrent_info> t) {
 
 QString misc::truncateRootFolder(libtorrent::torrent_handle h) {
   torrent_info t = h.get_torrent_info();
+#if LIBTORRENT_VERSION_MINOR >= 16
+  file_storage fs = t.files();
+#endif
   if(t.num_files() == 1) {
     // Single file torrent
     // Remove possible subfolders
+#if LIBTORRENT_VERSION_MINOR >= 16
+    QString path = misc::toQStringU(fs.file_path(t.file_at(0)));
+#else
     QString path = QString::fromUtf8(t.file_at(0).path.string().c_str());
-    QStringList path_parts = path.split("/", QString::SkipEmptyParts);
-    t.rename_file(0, path_parts.last().toUtf8().data());
+#endif
+    t.rename_file(0, fileName(path).toUtf8().data());
     return QString::null;
   }
   QString root_folder;
   int i = 0;
   for(torrent_info::file_iterator it = t.begin_files(); it < t.end_files(); it++) {
+#if LIBTORRENT_VERSION_MINOR >= 16
+    QString path = toQStringU(fs.file_path(*it));
+#else
     QString path = QString::fromUtf8(it->path.string().c_str());
+#endif
     QStringList path_parts = path.split("/", QString::SkipEmptyParts);
     if(path_parts.size() > 1) {
       root_folder = path_parts.takeFirst();
@@ -312,25 +358,24 @@ QString misc::truncateRootFolder(libtorrent::torrent_handle h) {
   return root_folder;
 }
 
-bool misc::sameFiles(QString path1, QString path2) {
-  QFile f1(path1);
-  if(!f1.exists()) return false;
-  QFile f2(path2);
-  if(!f2.exists()) return false;
-  QByteArray content1, content2;
-  if(f1.open(QIODevice::ReadOnly)) {
-    content1 = f1.readAll();
+bool misc::sameFiles(const QString &path1, const QString &path2) {
+  QFile f1(path1), f2(path2);
+  if(!f1.exists() || !f2.exists()) return false;
+  if(f1.size() != f2.size()) return false;
+  if(!f1.open(QIODevice::ReadOnly)) return false;
+  if(!f2.open(QIODevice::ReadOnly)) {
     f1.close();
-  } else {
     return false;
   }
-  if(f2.open(QIODevice::ReadOnly)) {
-    content1 = f2.readAll();
-    f2.close();
-  } else {
-    return false;
+  bool same = true;
+  while(!f1.atEnd() && !f2.atEnd()) {
+    if(f1.read(5) != f2.read(5)) {
+      same = false;
+      break;
+    }
   }
-  return content1 == content2;
+  f1.close(); f2.close();
+  return same;
 }
 
 void misc::copyDir(QString src_path, QString dst_path) {
@@ -363,12 +408,31 @@ void misc::copyDir(QString src_path, QString dst_path) {
   }
 }
 
-QString misc::updateLabelInSavePath(const QString& defaultSavePath, QString save_path, const QString old_label, const QString new_label) {
+void misc::chmod644(const QDir& folder) {
+  qDebug("chmod644(%s)", qPrintable(folder.absolutePath()));
+  if(!folder.exists()) return;
+  foreach(const QFileInfo &fi, folder.entryInfoList(QDir::Dirs|QDir::Files|QDir::NoSymLinks)) {
+    if(fi.fileName().startsWith(".")) continue;
+    if(fi.isDir()) {
+      misc::chmod644(QDir(fi.absoluteFilePath()));
+    } else {
+      QFile f(fi.absoluteFilePath());
+      f.setPermissions(f.permissions()|QFile::ReadUser|QFile::WriteUser|QFile::ReadGroup|QFile::ReadOther);
+    }
+  }
+}
+
+QString misc::updateLabelInSavePath(QString defaultSavePath, QString save_path, const QString &old_label, const QString &new_label) {
   if(old_label == new_label) return save_path;
+#if defined(Q_WS_WIN) || defined(Q_OS_OS2)
+  defaultSavePath.replace("\\", "/");
+  save_path.replace("\\", "/");
+#endif
   qDebug("UpdateLabelInSavePath(%s, %s, %s)", qPrintable(save_path), qPrintable(old_label), qPrintable(new_label));
   if(!save_path.startsWith(defaultSavePath)) return save_path;
-  QString new_save_path = save_path.replace(defaultSavePath, "");
-  QStringList path_parts = new_save_path.split(QDir::separator(), QString::SkipEmptyParts);
+  QString new_save_path = save_path;
+  new_save_path.replace(defaultSavePath, "");
+  QStringList path_parts = new_save_path.split("/", QString::SkipEmptyParts);
   if(path_parts.empty()) {
     if(!new_label.isEmpty())
       path_parts << new_label;
@@ -392,33 +456,6 @@ QString misc::updateLabelInSavePath(const QString& defaultSavePath, QString save
   return new_save_path;
 }
 
-void misc::moveToXDGFolders() {
-  QDir old_qBtPath(QDir::homePath().replace("\\","/")+"/"+".qbittorrent");
-  if(old_qBtPath.exists()) {
-    // Copy BT_backup folder
-    const QString old_BTBackupPath = old_qBtPath.absoluteFilePath("BT_backup");
-    if(QDir(old_BTBackupPath).exists()) {
-      copyDir(old_BTBackupPath, BTBackupLocation());
-    }
-    // Copy search engine folder
-    const QString old_searchPath = old_qBtPath.absoluteFilePath("search_engine");
-    if(QDir(old_searchPath).exists()) {
-      copyDir(old_searchPath, searchEngineLocation());
-    }
-    // Copy *_state files
-    if(QFile::exists(old_qBtPath.absoluteFilePath("dht_state"))) {
-      QFile::copy(old_qBtPath.absoluteFilePath("dht_state"), QDir(cacheLocation()).absoluteFilePath("dht_state"));
-      safeRemove(old_qBtPath.absoluteFilePath("dht_state"));
-    }
-    if(QFile::exists(old_qBtPath.absoluteFilePath("ses_state"))) {
-      QFile::copy(old_qBtPath.absoluteFilePath("ses_state"), QDir(cacheLocation()).absoluteFilePath("ses_state"));
-      safeRemove(old_qBtPath.absoluteFilePath("ses_state"));
-    }
-    // Remove .qbittorrent folder if empty
-    QDir::home().rmdir(".qbittorrent");
-  }
-}
-
 QString misc::toValidFileSystemName(QString filename) {
   qDebug("toValidFSName: %s", qPrintable(filename));
   filename = filename.replace("\\", "/").trimmed();
@@ -457,7 +494,7 @@ QPoint misc::screenCenter(QWidget *win) {
 
 QString misc::searchEngineLocation() {
   const QString location = QDir::cleanPath(QDesktopServicesDataLocation()
-                                           + QDir::separator() + "search_engine");
+                                           + QDir::separator() + "nova");
   QDir locationDir(location);
   if(!locationDir.exists())
     locationDir.mkpath(locationDir.absolutePath());
@@ -485,19 +522,19 @@ QString misc::cacheLocation() {
 // use Binary prefix standards from IEC 60027-2
 // see http://en.wikipedia.org/wiki/Kilobyte
 // value must be given in bytes
-QString misc::friendlyUnit(double val) {
+QString misc::friendlyUnit(qreal val) {
   if(val < 0)
     return tr("Unknown", "Unknown (size)");
-  const QString units[5] = {tr("B", "bytes"), tr("KiB", "kibibytes (1024 bytes)"), tr("MiB", "mebibytes (1024 kibibytes)"), tr("GiB", "gibibytes (1024 mibibytes)"), tr("TiB", "tebibytes (1024 gibibytes)")};
-  char i = 0;
+  int i = 0;
   while(val >= 1024. && i++<6)
     val /= 1024.;
   if(i == 0)
-    return QString::number((long)val) + " " + units[0];
-  return QString::number(val, 'f', 1) + " " + units[(int)i];
+    return QString::number((long)val) + " " + tr(units[0].source, units[0].comment);
+  return QString::number(val, 'f', 1) + " " + tr(units[i].source, units[i].comment);
 }
 
 bool misc::isPreviewable(QString extension){
+  if(extension.isEmpty()) return false;
   extension = extension.toUpper();
   if(extension == "AVI") return true;
   if(extension == "MP3") return true;
@@ -544,19 +581,6 @@ bool misc::isPreviewable(QString extension){
   return false;
 }
 
-bool misc::removeEmptyTree(QString path) {
-  QDir dir(path);
-  foreach(const QString &child, dir.entryList(QDir::AllDirs)) {
-    if(child == "." || child == "..") continue;
-    return removeEmptyTree(dir.absoluteFilePath(child));
-  }
-  const QString dir_name = dir.dirName();
-  if(dir.cdUp()) {
-    return dir.rmdir(dir_name);
-  }
-  return false;
-}
-
 QString misc::bcLinkToMagnet(QString bc_link) {
   QByteArray raw_bc = bc_link.toUtf8();
   raw_bc = raw_bc.mid(8); // skip bc://bt/
@@ -610,7 +634,7 @@ QString misc::magnetUriToHash(QString magnet_uri) {
   return hash;
 }
 
-QString misc::boostTimeToQString(const boost::optional<boost::posix_time::ptime> boostDate) {
+QString misc::boostTimeToQString(const boost::optional<boost::posix_time::ptime> &boostDate) {
   if(!boostDate || !boostDate.is_initialized() || boostDate->is_not_a_date_time()) return tr("Unknown");
   struct std::tm tm;
   try {
@@ -618,10 +642,18 @@ QString misc::boostTimeToQString(const boost::optional<boost::posix_time::ptime>
   } catch(std::exception e) {
     return tr("Unknown");
   }
-  time_t t = mktime(&tm);
+  const time_t t = mktime(&tm);
   if(t < 0)
     return tr("Unknown");
-  QDateTime dt = QDateTime::fromTime_t(t);
+  const QDateTime dt = QDateTime::fromTime_t(t);
+  if(dt.isNull() || !dt.isValid())
+    return tr("Unknown");
+  return dt.toString(Qt::DefaultLocaleLongDate);
+}
+
+QString misc::time_tToQString(const boost::optional<time_t> &t) {
+  if(!t.is_initialized() || *t < 0) return tr("Unknown");
+  QDateTime dt = QDateTime::fromTime_t(*t);
   if(dt.isNull() || !dt.isValid())
     return tr("Unknown");
   return dt.toString(Qt::DefaultLocaleLongDate);
@@ -689,10 +721,7 @@ QString misc::getUserIDString() {
 QStringList misc::toStringList(const QList<bool> &l) {
   QStringList ret;
   foreach(const bool &b, l) {
-    if(b)
-      ret << "1";
-    else
-      ret << "0";
+    ret << (b ? "1" : "0");
   }
   return ret;
 }
@@ -708,10 +737,75 @@ QList<int> misc::intListfromStringList(const QStringList &l) {
 QList<bool> misc::boolListfromStringList(const QStringList &l) {
   QList<bool> ret;
   foreach(const QString &s, l) {
-    if(s == "1")
-      ret << true;
-    else
-      ret << false;
+    ret << (s=="1");
   }
   return ret;
 }
+
+quint64 misc::computePathSize(QString path)
+{
+  // Check if it is a file
+  QFileInfo fi(path);
+  if(!fi.exists()) return 0;
+  if(fi.isFile()) return fi.size();
+  // Compute folder size
+  quint64 size = 0;
+  foreach(const QFileInfo &subfi, QDir(path).entryInfoList(QDir::Dirs|QDir::Files)) {
+    if(subfi.fileName().startsWith(".")) continue;
+    if(subfi.isDir())
+      size += misc::computePathSize(subfi.absoluteFilePath());
+    else
+      size += subfi.size();
+  }
+  return size;
+}
+
+bool misc::isValidTorrentFile(const QString &torrent_path) {
+  try {
+    boost::intrusive_ptr<libtorrent::torrent_info> t = new torrent_info(torrent_path.toUtf8().constData());
+    if(!t->is_valid() || t->num_files() == 0)
+      throw std::exception();
+  } catch(std::exception&) {
+    return false;
+  }
+  return true;
+}
+
+QString misc::branchPath(QString file_path, bool uses_slashes)
+{
+  if(!uses_slashes)
+    file_path.replace("\\", "/");
+  Q_ASSERT(!file_path.contains("\\"));
+  if(file_path.endsWith("/"))
+    file_path.chop(1); // Remove trailing slash
+  qDebug() << Q_FUNC_INFO << "before:" << file_path;
+  if(file_path.contains("/"))
+    return file_path.left(file_path.lastIndexOf('/'));
+  return "";
+}
+
+bool misc::isUrl(const QString &s)
+{
+  const QString scheme = QUrl(s).scheme();
+  QRegExp is_url("http[s]?|ftp", Qt::CaseInsensitive);
+  return is_url.exactMatch(scheme);
+}
+
+QString misc::fileName(QString file_path)
+{
+  file_path.replace("\\", "/");
+  const int slash_index = file_path.lastIndexOf('/');
+  if(slash_index == -1)
+    return file_path;
+  return file_path.mid(slash_index+1);
+}
+
+bool misc::removeEmptyFolder(const QString &dirpath)
+{
+  QDir savedir(dirpath);
+  const QString dirname = savedir.dirName();
+  if(savedir.exists() && savedir.cdUp()) {
+    return savedir.rmdir(dirname);
+  }
+  return false;
+}
