@@ -40,41 +40,52 @@
 #include <boost/date_time/posix_time/conversion.hpp>
 #include <QPoint>
 #include <QFile>
+#include <QDir>
+#ifndef DISABLE_GUI
+#include <QIcon>
+#endif
 
 #include <libtorrent/torrent_info.hpp>
 #include <libtorrent/torrent_handle.hpp>
 
 const qlonglong MAX_ETA = 8640000;
 
-using namespace libtorrent;
-
 /*  Miscellaneaous functions that can be useful */
 class misc : public QObject{
   Q_OBJECT
 
+private:
+  misc(); // Forbidden
+
 public:
-  static inline QString toQString(std::string str) {
+  static inline QString toQString(const std::string &str) {
     return QString::fromLocal8Bit(str.c_str());
   }
 
-  static inline QString toQString(char* str) {
+  static inline QString toQString(const char* str) {
     return QString::fromLocal8Bit(str);
   }
 
-  static inline QString toQStringU(std::string str) {
+  static inline QString toQStringU(const std::string &str) {
     return QString::fromUtf8(str.c_str());
   }
 
-  static inline QString toQStringU(char* str) {
+  static inline QString toQStringU(const char* str) {
     return QString::fromUtf8(str);
   }
 
-  static inline QString toQString(sha1_hash hash) {
+  static inline QString toQString(const libtorrent::sha1_hash &hash) {
     std::ostringstream o;
     o << hash;
     return QString(o.str().c_str());
   }
 
+  static inline libtorrent::sha1_hash toSha1Hash(const QString &hash) {
+    return libtorrent::sha1_hash(qPrintable(hash));
+  }
+
+  static void chmod644(const QDir& folder);
+
   static inline QString removeLastPathPart(QString path) {
     if(path.isEmpty()) return path;
     path = path.replace("\\", "/");
@@ -83,15 +94,25 @@ public:
     return tmp.join("/");
   }
 
-  static inline sha1_hash QStringToSha1(const QString& s) {
+  static inline libtorrent::sha1_hash QStringToSha1(const QString& s) {
     std::string str(s.toLocal8Bit().data());
     std::istringstream i(str);
-    sha1_hash x;
+    libtorrent::sha1_hash x;
     i>>x;
     return x;
   }
 
-  static void shutdownComputer();
+  static inline QString file_extension(const QString &filename) {
+    QString extension;
+    if(filename.contains(".")) {
+      extension = filename.mid(filename.lastIndexOf(".")+1);
+    }
+    return extension;
+  }
+
+#ifndef DISABLE_GUI
+  static void shutdownComputer(bool sleep=false);
+#endif
 
   static bool safeRemove(QString file_path) {
     QFile MyFile(file_path);
@@ -102,16 +123,18 @@ public:
     return MyFile.remove();
   }
 
-  static QString truncateRootFolder(boost::intrusive_ptr<torrent_info> t);
-  static QString truncateRootFolder(torrent_handle h);
+  static bool removeEmptyFolder(const QString &dirpath);
 
-  static QString updateLabelInSavePath(const QString& defaultSavePath, QString save_path, const QString old_label, const QString new_label);
+  static quint64 computePathSize(QString path);
 
-  static bool sameFiles(QString path1, QString path2);
+  static QString truncateRootFolder(boost::intrusive_ptr<libtorrent::torrent_info> t);
+  static QString truncateRootFolder(libtorrent::torrent_handle h);
+
+  static QString updateLabelInSavePath(QString defaultSavePath, QString save_path, const QString &old_label, const QString &new_label);
+
+  static bool sameFiles(const QString &path1, const QString &path2);
+  static bool isUrl(const QString &s);
   static void copyDir(QString src_path, QString dst_path);
-  // Introduced in v2.1.0 for backward compatibility
-  // Remove after some releases
-  static void moveToXDGFolders();
   static QString toValidFileSystemName(QString filename);
   static bool isValidFileSystemName(QString filename);
 
@@ -132,13 +155,15 @@ public:
   // use Binary prefix standards from IEC 60027-2
   // see http://en.wikipedia.org/wiki/Kilobyte
   // value must be given in bytes
-  static QString friendlyUnit(double val);
+  static QString friendlyUnit(qreal val);
   static bool isPreviewable(QString extension);
-  static bool removeEmptyTree(QString path);
+  static QString branchPath(QString file_path, bool uses_slashes=false);
+  static QString fileName(QString file_path);
   static QString magnetUriToName(QString magnet_uri);
   static QString magnetUriToHash(QString magnet_uri);
   static QString bcLinkToMagnet(QString bc_link);
-  static QString boostTimeToQString(const boost::optional<boost::posix_time::ptime> boostDate);
+  static QString boostTimeToQString(const boost::optional<boost::posix_time::ptime> &boostDate);
+  static QString time_tToQString(const boost::optional<time_t> &t);
   // Replace ~ in path
   static QString expandPath(QString path);
   // Take a number of seconds and return an user-friendly
@@ -150,6 +175,8 @@ public:
   static QStringList toStringList(const QList<bool> &l);
   static QList<int> intListfromStringList(const QStringList &l);
   static QList<bool> boolListfromStringList(const QStringList &l);
+
+  static bool isValidTorrentFile(const QString &path);
 };
 
 //  Trick to get a portable sleep() function
