@@ -14,9 +14,11 @@
 
 #include <QString>
 #include <QObject>
+#include <QByteArray>
 
 struct QTextStream;
 struct QTcpSocket;
+class QTextCodec;
 
 class Smtp : public QObject {
   Q_OBJECT
@@ -29,13 +31,16 @@ private slots:
   void readyRead();
 
 private:
-  QString message;
+  QByteArray encode_mime_header(const QString& key, const QString& value, QTextCodec* latin1, const QByteArray& prefix=QByteArray());
+
+private:
+  QByteArray message;
   QTextStream *t;
   QTcpSocket *socket;
   QString from;
   QString rcpt;
   QString response;
-  enum states{Rcpt,Mail,Data,Init,Body,Quit,Close};
+  enum states{Rcpt,Mail,Mail2,Data,Init,Body,Quit,Close};
   int state;
 
 };
