@@ -31,41 +31,21 @@
 #ifndef TORRENTADDITION_H
 #define TORRENTADDITION_H
 
-#include <QDir>
-#include <QFileDialog>
-#include <QFile>
-#include <fstream>
-#include <QMessageBox>
-#include <QMenu>
-#include <QHeaderView>
-#include <QApplication>
-#include <QDesktopWidget>
-#include <QInputDialog>
+#include <QStringList>
 
-#include <libtorrent/version.hpp>
-#include <libtorrent/session.hpp>
-#include <libtorrent/bencode.hpp>
-#include "bittorrent.h"
-#include "misc.h"
-#include "proplistdelegate.h"
 #include "ui_torrentadditiondlg.h"
-#include "torrentpersistentdata.h"
-#include "torrentfilesmodel.h"
-#include "preferences.h"
-#include "GUI.h"
-#include "transferlistwidget.h"
-#include "qinisettings.h"
 
-using namespace libtorrent;
+#include <libtorrent/torrent_info.hpp>
+
+class TorrentFilesModel;
+class PropListDelegate;
 
 class torrentAdditionDialog : public QDialog, private Ui_addTorrentDialog{
   Q_OBJECT
 
 public:
-  torrentAdditionDialog(GUI *parent, Bittorrent* _BTSession);
+  torrentAdditionDialog(QWidget *parent);
   ~torrentAdditionDialog();
-  void readSettings();
-  void saveSettings();
   void showLoadMagnetURI(QString magnet_uri);
   void showLoad(QString filePath, QString from_url=QString::null);
   QString getCurrentTruncatedSavePath(QString* root_folder_or_file_name = 0) const;
@@ -86,17 +66,19 @@ public slots:
   void saveTruncatedPathHistory();
   void loadSavePathHistory();
   void updateLabelInSavePath(QString label);
-  void updateSavePathCurrentText(QString path);
+  void updateSavePathCurrentText();
   void resetComboLabelIndex(QString text);
 
-protected slots:
-  void restoreCursorPosition();
+protected:
+  void closeEvent(QCloseEvent *event);
 
-signals:
-  void torrentPaused(QTorrentHandle &h);
+private:
+  void readSettings();
+  void saveSettings();
+  void loadFilesListState();
+  void saveFilesListState();
 
 private:
-  Bittorrent *BTSession;
   QString fileName;
   QString hash;
   QString filePath;
@@ -107,12 +89,11 @@ private:
   TorrentFilesModel *PropListModel;
   PropListDelegate *PropDelegate;
   unsigned int nbFiles;
-  boost::intrusive_ptr<torrent_info> t;
+  boost::intrusive_ptr<libtorrent::torrent_info> t;
   QStringList files_path;
   bool is_magnet;
   int hidden_height;
   QStringList path_history;
-  int cursor_pos;
 };
 
 #endif
