@@ -39,29 +39,39 @@
 #include <QDir>
 #include <libtorrent/torrent_info.hpp>
 #include "proplistdelegate.h"
-#include "misc.h"
-using namespace libtorrent;
+#include "iconprovider.h"
 
-enum FilePriority {IGNORED=0, NORMAL=1, HIGH=2, MAXIMUM=7};
-enum TreeItemType {TFILE, FOLDER, ROOT};
+namespace prio {
+enum FilePriority {IGNORED=0, NORMAL=1, HIGH=2, MAXIMUM=7, PARTIAL=-1};
+}
+
+class TorrentFileItem {
+
+public:
+  enum TreeItemColumns {COL_NAME, COL_SIZE, COL_PROGRESS, COL_PRIO};
+  enum FileType {TFILE, FOLDER, ROOT};
 
-class TreeItem {
 private:
-  QList<TreeItem*> childItems;
+  QList<TorrentFileItem*> childItems;
   QList<QVariant> itemData;
-  TreeItem *parentItem;
-  TreeItemType type;
+  TorrentFileItem *parentItem;
+  FileType m_type;
   qulonglong total_done;
   int file_index;
 
 public:
   // File Construction
-  TreeItem(file_entry f, TreeItem *parent, int _file_index) {
+  TorrentFileItem(const libtorrent::torrent_info &t, const libtorrent::file_entry &f, TorrentFileItem *parent, int _file_index) {
     Q_ASSERT(parent);
     parentItem = parent;
-    type = TFILE;
+    m_type = TFILE;
     file_index = _file_index;
-    QString name = misc::toQStringU(f.path.string()).split("/").last();
+#if LIBTORRENT_VERSION_MINOR >= 16
+    QString name = misc::fileName(misc::toQStringU(t.files().file_path(f)));
+#else
+    Q_UNUSED(t);
+    QString name = misc::toQStringU(f.path.filename());
+#endif
     // Do not display incomplete extensions
     if(name.endsWith(".!qB"))
       name.chop(4);
@@ -71,7 +81,7 @@ public:
     itemData << QVariant((qulonglong)f.size);
     total_done = 0;
     itemData << 0.; // Progress;
-    itemData << NORMAL; // Priority
+    itemData << prio::NORMAL; // Priority
     if(parent) {
       parent->appendChild(this);
       parent->updateSize();
@@ -79,9 +89,9 @@ public:
   }
 
   // Folder constructor
-  TreeItem(QString name, TreeItem *parent=0) {
+  TorrentFileItem(QString name, TorrentFileItem *parent=0) {
     parentItem = parent;
-    type = FOLDER;
+    m_type = FOLDER;
     // Do not display incomplete extensions
     if(name.endsWith(".!qB"))
       name.chop(4);
@@ -89,88 +99,89 @@ public:
     itemData << 0.; // Size
     itemData << 0.; // Progress;
     total_done = 0;
-    itemData << NORMAL; // Priority
+    itemData << prio::NORMAL; // Priority
     if(parent) {
       parent->appendChild(this);
     }
   }
 
-  TreeItem(QList<QVariant> data) {
+  TorrentFileItem(QList<QVariant> data) {
     parentItem = 0;
-    type = ROOT;
+    m_type = ROOT;
     Q_ASSERT(data.size() == 4);
     itemData = data;
     total_done = 0;
   }
 
-  ~TreeItem() {
+  ~TorrentFileItem() {
     qDebug("Deleting item: %s", qPrintable(getName()));
     qDeleteAll(childItems);
   }
 
-  TreeItemType getType() const {
-    return type;
+  FileType getType() const {
+    return m_type;
   }
 
   int getFileIndex() const {
-    Q_ASSERT(type ==TFILE);
+    Q_ASSERT(m_type ==TFILE);
     return file_index;
   }
 
   void deleteAllChildren() {
-    Q_ASSERT(type==ROOT);
+    Q_ASSERT(m_type==ROOT);
     qDeleteAll(childItems);
     childItems.clear();
     Q_ASSERT(childItems.empty());
   }
 
-  const QList<TreeItem*>& children() const {
+  QList<TorrentFileItem*> children() const {
     return childItems;
   }
 
   QString getName() const {
     //Q_ASSERT(type != ROOT);
-    return itemData.first().toString();
+    return itemData.at(COL_NAME).toString();
   }
 
   void setName(QString name) {
-    Q_ASSERT(type != ROOT);
-    itemData.replace(0, name);
+    Q_ASSERT(m_type != ROOT);
+    itemData.replace(COL_NAME, name);
   }
 
   qulonglong getSize() const {
-    return itemData.value(1).toULongLong();
+    return itemData.value(COL_SIZE).toULongLong();
   }
 
   void setSize(qulonglong size) {
     if(getSize() == size) return;
-    itemData.replace(1, (qulonglong)size);
+    itemData.replace(COL_SIZE, (qulonglong)size);
     if(parentItem)
       parentItem->updateSize();
   }
 
   void updateSize() {
-    if(type == ROOT) return;
-    Q_ASSERT(type == FOLDER);
+    if(m_type == ROOT) return;
+    Q_ASSERT(m_type == FOLDER);
     qulonglong size = 0;
-    foreach(TreeItem* child, childItems) {
-      if(child->getPriority() > 0)
+    foreach(TorrentFileItem* child, childItems) {
+      if(child->getPriority() != prio::IGNORED)
         size += child->getSize();
     }
     setSize(size);
   }
 
   void setProgress(qulonglong done) {
+    if(getPriority() == 0) return;
     total_done = done;
     qulonglong size = getSize();
     Q_ASSERT(total_done <= size);
-    float progress;
+    qreal progress;
     if(size > 0)
       progress = total_done/(float)size;
     else
       progress = 1.;
     Q_ASSERT(progress >= 0. && progress <= 1.);
-    itemData.replace(2, progress);
+    itemData.replace(COL_PROGRESS, progress);
     if(parentItem)
       parentItem->updateProgress();
   }
@@ -179,7 +190,9 @@ public:
     return total_done;
   }
 
-  float getProgress() const {
+  qreal getProgress() const {
+    if(getPriority() == 0)
+      return 0.;
     qulonglong size = getSize();
     if(size > 0)
       return total_done/(float)getSize();
@@ -187,10 +200,10 @@ public:
   }
 
   void updateProgress() {
-    if(type == ROOT) return;
-    Q_ASSERT(type == FOLDER);
+    if(m_type == ROOT) return;
+    Q_ASSERT(m_type == FOLDER);
     total_done = 0;
-    foreach(TreeItem* child, childItems) {
+    foreach(TorrentFileItem* child, childItems) {
       if(child->getPriority() > 0)
         total_done += child->getTotalDone();
     }
@@ -200,67 +213,80 @@ public:
   }
 
   int getPriority() const {
-    return itemData.value(3).toInt();
+    return itemData.value(COL_PRIO).toInt();
   }
 
-  void setPriority(int new_prio, bool update_children=true, bool update_parent=true) {
-    int old_prio = getPriority();
-    if(old_prio != new_prio) {
-      qDebug("setPriority(%s, %d)", qPrintable(getName()), new_prio);
-      itemData.replace(3, new_prio);
-      // Update parent
-      if(update_parent && parentItem) {
-        parentItem->updateSize();
-        parentItem->updateProgress();
-        parentItem->updatePriority();
-      }
+  void setPriority(int new_prio, bool update_parent=true) {
+    Q_ASSERT(new_prio != prio::PARTIAL || m_type == FOLDER); // PARTIAL only applies to folders
+    const int old_prio = getPriority();
+    if(old_prio == new_prio) return;
+    qDebug("setPriority(%s, %d)", qPrintable(getName()), new_prio);
+    // Reset progress if priority is 0
+    if(new_prio == 0) {
+      setProgress(0);
     }
+
+    itemData.replace(COL_PRIO, new_prio);
+
+    // Update parent
+    if(update_parent && parentItem) {
+      qDebug("Updating parent item");
+      parentItem->updateSize();
+      parentItem->updateProgress();
+      parentItem->updatePriority();
+    }
+
     // Update children
-    if(update_children) {
-      foreach(TreeItem* child, childItems) {
-        child->setPriority(new_prio, true, false);
+    if(new_prio != prio::PARTIAL && !childItems.empty()) {
+      qDebug("Updating children items");
+      foreach(TorrentFileItem* child, childItems) {
+        // Do not update the parent since
+        // the parent is causing the update
+        child->setPriority(new_prio, false);
       }
     }
-    if(type==FOLDER) {
+    if(m_type == FOLDER) {
       updateSize();
       updateProgress();
     }
   }
 
+  // Only non-root folders use this function
   void updatePriority() {
-    if(type == ROOT) return;
-    Q_ASSERT(type == FOLDER);
-    int priority = getPriority();
-    bool first = true;
-    foreach(TreeItem* child, childItems) {
-      if(first) {
-        priority = child->getPriority();
-        first = false;
-      } else {
-        if(child->getPriority() != priority) {
-          setPriority(NORMAL, false);
-          return;
-        }
+    if(m_type == ROOT) return;
+    Q_ASSERT(m_type == FOLDER);
+    if(childItems.isEmpty()) return;
+    // If all children have the same priority
+    // then the folder should have the same
+    // priority
+    const int prio = childItems.first()->getPriority();
+    for(int i=1; i<childItems.size(); ++i) {
+      if(childItems.at(i)->getPriority() != prio) {
+        setPriority(prio::PARTIAL);
+        return;
       }
     }
-    setPriority(priority);
+    // All child items have the same priorrity
+    // Update mine if necessary
+    if(prio != getPriority())
+      setPriority(prio);
   }
 
-  TreeItem* childWithName(QString name) const {
-    foreach(TreeItem *child, childItems) {
+  TorrentFileItem* childWithName(QString name) const {
+    foreach(TorrentFileItem *child, childItems) {
       if(child->getName() == name) return child;
     }
     return 0;
   }
 
   bool isFolder() const {
-    return (type==FOLDER);
+    return (m_type==FOLDER);
   }
 
-  void appendChild(TreeItem *item) {
+  void appendChild(TorrentFileItem *item) {
     Q_ASSERT(item);
     //Q_ASSERT(!childWithName(item->getName()));
-    Q_ASSERT(type != TFILE);
+    Q_ASSERT(m_type != TFILE);
     int i=0;
     for(i=0; i<childItems.size(); ++i) {
       QString newchild_name = item->getName();
@@ -271,7 +297,7 @@ public:
     //Q_ASSERT(type != ROOT || childItems.size() == 1);
   }
 
-  TreeItem *child(int row) {
+  TorrentFileItem *child(int row) {
     //Q_ASSERT(row >= 0 && row < childItems.size());
     return childItems.value(row, 0);
   }
@@ -290,12 +316,12 @@ public:
 
   int row() const {
     if (parentItem) {
-      return parentItem->children().indexOf(const_cast<TreeItem*>(this));
+      return parentItem->children().indexOf(const_cast<TorrentFileItem*>(this));
     }
     return 0;
   }
 
-  TreeItem *parent() {
+  TorrentFileItem *parent() {
     return parentItem;
   }
 };
@@ -305,15 +331,15 @@ class TorrentFilesModel:  public QAbstractItemModel {
   Q_OBJECT
 
 private:
-  TreeItem *rootItem;
-  TreeItem **files_index;
+  TorrentFileItem *rootItem;
+  TorrentFileItem **files_index;
 
 public:
   TorrentFilesModel(QObject *parent=0): QAbstractItemModel(parent) {
     files_index = 0;
     QList<QVariant> rootData;
     rootData << tr("Name") << tr("Size") << tr("Progress") << tr("Priority");
-    rootItem = new TreeItem(rootData);
+    rootItem = new TorrentFileItem(rootData);
   }
 
   ~TorrentFilesModel() {
@@ -321,7 +347,7 @@ public:
     delete rootItem;
   }
 
-  void updateFilesProgress(std::vector<size_type> fp) {
+  void updateFilesProgress(std::vector<libtorrent::size_type> fp) {
     for(unsigned int i=0; i<fp.size(); ++i) {
       Q_ASSERT(fp[i] >= 0);
       files_index[i]->setProgress(fp[i]);
@@ -348,7 +374,7 @@ public:
 
   bool allFiltered() const {
     for(int i=0; i<rootItem->childCount(); ++i) {
-      if(rootItem->child(i)->getPriority() != IGNORED)
+      if(rootItem->child(i)->getPriority() != prio::IGNORED)
         return false;
     }
     return true;
@@ -356,38 +382,41 @@ public:
 
   int columnCount(const QModelIndex &parent=QModelIndex()) const {
     if (parent.isValid())
-      return static_cast<TreeItem*>(parent.internalPointer())->columnCount();
+      return static_cast<TorrentFileItem*>(parent.internalPointer())->columnCount();
     else
       return rootItem->columnCount();
   }
 
   bool setData(const QModelIndex & index, const QVariant & value, int role = Qt::EditRole) {
     if(!index.isValid()) return false;
-    if (role == Qt::CheckStateRole) {
-      TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
+    if (index.column() == 0 && role == Qt::CheckStateRole) {
+      TorrentFileItem *item = static_cast<TorrentFileItem*>(index.internalPointer());
+      qDebug("setData(%s, %d", qPrintable(item->getName()), value.toInt());
       if(item->getPriority() != value.toInt()) {
-        if(value.toInt() == Qt::Checked)
-          item->setPriority(NORMAL);
+        if(value.toInt() == Qt::PartiallyChecked)
+          item->setPriority(prio::PARTIAL);
+        else if (value.toInt() == Qt::Unchecked)
+          item->setPriority(prio::IGNORED);
         else
-          item->setPriority(IGNORED);
+          item->setPriority(prio::NORMAL);
         emit filteredFilesChanged();
         emit dataChanged(this->index(0,0), this->index(rowCount(), 0));
       }
       return true;
     }
     if (role == Qt::EditRole) {
-      TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
+      TorrentFileItem *item = static_cast<TorrentFileItem*>(index.internalPointer());
       switch(index.column()) {
-      case 0:
+      case TorrentFileItem::COL_NAME:
         item->setName(value.toString());
         break;
-      case 1:
+      case TorrentFileItem::COL_SIZE:
         item->setSize(value.toULongLong());
         break;
-      case 2:
+      case TorrentFileItem::COL_PROGRESS:
         item->setProgress(value.toDouble());
         break;
-      case 3:
+      case TorrentFileItem::COL_PRIO:
         item->setPriority(value.toInt());
         break;
       default:
@@ -399,29 +428,31 @@ public:
     return false;
   }
 
-  TreeItemType getType(const QModelIndex &index) {
-    TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
+  TorrentFileItem::FileType getType(const QModelIndex &index) const {
+    const TorrentFileItem *item = static_cast<const TorrentFileItem*>(index.internalPointer());
     return item->getType();
   }
 
   int getFileIndex(const QModelIndex &index) {
-    TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
+    TorrentFileItem *item = static_cast<TorrentFileItem*>(index.internalPointer());
     return item->getFileIndex();
   }
 
   QVariant data(const QModelIndex &index, int role=Qt::DisplayRole) const {
     if (!index.isValid())
       return QVariant();
-    TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
+    TorrentFileItem *item = static_cast<TorrentFileItem*>(index.internalPointer());
     if(index.column() == 0 && role == Qt::DecorationRole) {
       if(item->isFolder())
-        return QIcon(":/Icons/oxygen/folder.png");
+        return IconProvider::instance()->getIcon("inode-directory");
       else
-        return QIcon(":/Icons/oxygen/file.png");
+        return IconProvider::instance()->getIcon("text-plain");
     }
-    if(role == Qt::CheckStateRole) {
-      if(item->data(3).toInt() == IGNORED)
+    if(index.column() == 0 && role == Qt::CheckStateRole) {
+      if(item->data(TorrentFileItem::COL_PRIO).toInt() == prio::IGNORED)
         return Qt::Unchecked;
+      if(item->data(TorrentFileItem::COL_PRIO).toInt() == prio::PARTIAL)
+        return Qt::PartiallyChecked;
       return Qt::Checked;
     }
     if (role != Qt::DisplayRole)
@@ -433,7 +464,8 @@ public:
   Qt::ItemFlags flags(const QModelIndex &index) const {
     if (!index.isValid())
       return 0;
-
+    if(getType(index) == TorrentFileItem::FOLDER)
+      return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsTristate;
     return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable;
   }
 
@@ -448,14 +480,14 @@ public:
     if (parent.isValid() && parent.column() != 0)
       return QModelIndex();
 
-    TreeItem *parentItem;
+    TorrentFileItem *parentItem;
 
     if (!parent.isValid())
       parentItem = rootItem;
     else
-      parentItem = static_cast<TreeItem*>(parent.internalPointer());
+      parentItem = static_cast<TorrentFileItem*>(parent.internalPointer());
 
-    TreeItem *childItem = parentItem->child(row);
+    TorrentFileItem *childItem = parentItem->child(row);
     if (childItem) {
       return createIndex(row, column, childItem);
     } else {
@@ -467,8 +499,8 @@ public:
     if (!index.isValid())
       return QModelIndex();
 
-    TreeItem *childItem = static_cast<TreeItem*>(index.internalPointer());
-    TreeItem *parentItem = childItem->parent();
+    TorrentFileItem *childItem = static_cast<TorrentFileItem*>(index.internalPointer());
+    TorrentFileItem *parentItem = childItem->parent();
 
     if (parentItem == rootItem)
       return QModelIndex();
@@ -477,7 +509,7 @@ public:
   }
 
   int rowCount(const QModelIndex &parent=QModelIndex()) const {
-    TreeItem *parentItem;
+    TorrentFileItem *parentItem;
 
     if (parent.column() > 0)
       return 0;
@@ -485,7 +517,7 @@ public:
     if (!parent.isValid())
       parentItem = rootItem;
     else
-      parentItem = static_cast<TreeItem*>(parent.internalPointer());
+      parentItem = static_cast<TorrentFileItem*>(parent.internalPointer());
 
     return parentItem->childCount();
   }
@@ -501,53 +533,40 @@ public:
     emit layoutChanged();
   }
 
-  void setupModelData(torrent_info const& t) {
+  void setupModelData(const libtorrent::torrent_info &t) {
     qDebug("setup model data called");
     if(t.num_files() == 0) return;
     // Initialize files_index array
-    files_index = new TreeItem*[t.num_files()];
+    qDebug("Torrent contains %d files", t.num_files());
+    files_index = new TorrentFileItem*[t.num_files()];
 
-    TreeItem *parent = this->rootItem;
-    /*if(t.num_files() == 1) {
-      // Create possible parent folder
-      QStringList path_parts = misc::toQStringU(t.file_at(0).path.string()).split("/", QString::SkipEmptyParts);
-      path_parts.removeLast();
-      foreach(const QString &part, path_parts) {
-        TreeItem *folder = new TreeItem(part, parent);
-        parent = folder;
-      }
-      TreeItem *f = new TreeItem(t.file_at(0), parent, 0);
-      //parent->appendChild(f);
-      files_index[0] = f;
-      emit layoutChanged();
-      return;
-    }*/
-    // Create parent folder
-    //QString root_name = misc::toQString(t.file_at(0).path.string()).split("/").first();
-    //TreeItem *current_parent = new TreeItem(root_name, parent);
-    //parent->appendChild(current_parent);
-    //TreeItem *root_folder = current_parent;
-    TreeItem *root_folder = parent;
-    TreeItem *current_parent;
+    TorrentFileItem *parent = this->rootItem;
+    TorrentFileItem *root_folder = parent;
+    TorrentFileItem *current_parent;
 
     // Iterate over files
     int i = 0;
-    torrent_info::file_iterator fi = t.begin_files();
+    libtorrent::torrent_info::file_iterator fi = t.begin_files();
     while(fi != t.end_files()) {
       current_parent = root_folder;
+#if LIBTORRENT_VERSION_MINOR >= 16
+      QString path = QDir::cleanPath(misc::toQStringU(t.files().file_path(*fi))).replace("\\", "/");
+#else
       QString path = QDir::cleanPath(misc::toQStringU(fi->path.string())).replace("\\", "/");
+#endif
       // Iterate of parts of the path to create necessary folders
       QStringList pathFolders = path.split("/");
+      pathFolders.removeAll(".unwanted");
       pathFolders.takeLast();
       foreach(const QString &pathPart, pathFolders) {
-        TreeItem *new_parent = current_parent->childWithName(pathPart);
+        TorrentFileItem *new_parent = current_parent->childWithName(pathPart);
         if(!new_parent) {
-          new_parent = new TreeItem(pathPart, current_parent);
+          new_parent = new TorrentFileItem(pathPart, current_parent);
         }
         current_parent = new_parent;
       }
       // Actually create the file
-      TreeItem *f = new TreeItem(*fi, current_parent, i);
+      TorrentFileItem *f = new TorrentFileItem(t, *fi, current_parent, i);
       files_index[i] = f;
       fi++;
       ++i;
@@ -558,21 +577,20 @@ public:
 public slots:
   void selectAll() {
     for(int i=0; i<rootItem->childCount(); ++i) {
-      TreeItem *child = rootItem->child(i);
-      if(child->getPriority() == IGNORED)
-        child->setPriority(NORMAL);
+      TorrentFileItem *child = rootItem->child(i);
+      if(child->getPriority() == prio::IGNORED)
+        child->setPriority(prio::NORMAL);
     }
     emit layoutChanged();
   }
 
   void selectNone() {
     for(int i=0; i<rootItem->childCount(); ++i) {
-      rootItem->child(i)->setPriority(IGNORED);
+      rootItem->child(i)->setPriority(prio::IGNORED);
     }
     emit layoutChanged();
   }
 
-public:
 signals:
   void filteredFilesChanged();
 };
