@@ -41,10 +41,14 @@
 #include <QDragMoveEvent>
 #include <QStandardItemModel>
 #include <QMessageBox>
+#include <QScrollBar>
 
 #include "transferlistdelegate.h"
 #include "transferlistwidget.h"
+#include "preferences.h"
 #include "qinisettings.h"
+#include "torrentmodel.h"
+#include "iconprovider.h"
 
 class LabelFiltersList: public QListWidget {
   Q_OBJECT
@@ -57,6 +61,7 @@ public:
     itemHover = 0;
     // Accept drop
     setAcceptDrops(true);
+    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
   }
 
   // Redefine addItem() to make sure the list stays sorted
@@ -133,11 +138,11 @@ protected:
   void setItemHover(bool hover) {
     Q_ASSERT(itemHover);
     if(hover) {
-      itemHover->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder-documents.png"));
+      itemHover->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("folder-documents.png"));
       itemHover->setSelected(true);
       //setCurrentItem(itemHover);
     } else {
-      itemHover->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder.png"));
+      itemHover->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("inode-directory.png"));
       //itemHover->setSelected(false);
       itemHover = 0;
     }
@@ -145,22 +150,43 @@ protected:
 };
 
 class StatusFiltersWidget : public QListWidget {
+  Q_OBJECT
+
 public:
-  StatusFiltersWidget(QWidget *parent) : QListWidget(parent) { 
-    setFixedHeight(120);
+  StatusFiltersWidget(QWidget *parent) : QListWidget(parent), m_shown(false) {
+    //setFixedHeight(120);
+    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
   }
+
+public slots:
+  void updateHeight() {
+    qDebug("Adjusting statuslist widget height...");
+    m_shown = true;
+    setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    int cur_height = 50;
+    do {
+      setFixedHeight(cur_height);
+      cur_height += 10;
+    }while(verticalScrollBar()->isVisible());
+    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  }
+
 protected:
   void changeEvent(QEvent *e) {
     QListWidget::changeEvent(e);
     switch (e->type()) {
     case QEvent::StyleChange:
-      setSpacing(0);
-      setFixedHeight(120);
+      qDebug("Style has changed, adapting the statusfilters widget height");
+      if(m_shown)
+        updateHeight();
       break;
     default:
       break;
     }
   }
+
+private:
+  bool m_shown;
   
 };
 
@@ -180,6 +206,7 @@ public:
   TransferListFiltersWidget(QWidget *parent, TransferListWidget *transferList): QFrame(parent), transferList(transferList), nb_labeled(0), nb_torrents(0) {
     // Construct lists
     vLayout = new QVBoxLayout();
+    vLayout->setContentsMargins(0, 4, 0, 4);
     statusFilters = new StatusFiltersWidget(this);
     vLayout->addWidget(statusFilters);
     labelFilters = new LabelFiltersList(this);
@@ -212,20 +239,20 @@ public:
 
     // SIGNAL/SLOT
     connect(statusFilters, SIGNAL(currentRowChanged(int)), transferList, SLOT(applyStatusFilter(int)));
-    connect(transferList, SIGNAL(torrentStatusUpdate(uint,uint,uint,uint,uint)), this, SLOT(updateTorrentNumbers(uint, uint, uint, uint, uint)));
+    connect(transferList->getSourceModel(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(updateTorrentNumbers()));
+    connect(transferList->getSourceModel(), SIGNAL(torrentAdded(TorrentModelItem*)), SLOT(handleNewTorrent(TorrentModelItem*)));
     connect(labelFilters, SIGNAL(currentRowChanged(int)), this, SLOT(applyLabelFilter(int)));
     connect(labelFilters, SIGNAL(torrentDropped(int)), this, SLOT(torrentDropped(int)));
-    connect(transferList, SIGNAL(torrentAdded(QModelIndex)), this, SLOT(torrentAdded(QModelIndex)));
-    connect(transferList, SIGNAL(torrentAboutToBeRemoved(QModelIndex)), this, SLOT(torrentAboutToBeDeleted(QModelIndex)));
-    connect(transferList, SIGNAL(torrentChangedLabel(QString,QString)), this, SLOT(torrentChangedLabel(QString, QString)));
+    connect(transferList->getSourceModel(), SIGNAL(torrentAboutToBeRemoved(TorrentModelItem*)), SLOT(torrentAboutToBeDeleted(TorrentModelItem*)));
+    connect(transferList->getSourceModel(), SIGNAL(torrentChangedLabel(TorrentModelItem*,QString,QString)), SLOT(torrentChangedLabel(TorrentModelItem*, QString, QString)));
 
     // Add Label filters
     QListWidgetItem *allLabels = new QListWidgetItem(labelFilters);
     allLabels->setData(Qt::DisplayRole, QVariant(tr("All labels") + " (0)"));
-    allLabels->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder.png"));
+    allLabels->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("inode-directory"));
     QListWidgetItem *noLabel = new QListWidgetItem(labelFilters);
     noLabel->setData(Qt::DisplayRole, QVariant(tr("Unlabeled") + " (0)"));
-    noLabel->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder.png"));
+    noLabel->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("inode-directory"));
 
     // Load settings
     loadSettings();
@@ -245,7 +272,7 @@ public:
     delete vLayout;
   }
 
-  QListWidget* getStatusFilters() const {
+  StatusFiltersWidget* getStatusFilters() const {
     return statusFilters;
   }
 
@@ -257,35 +284,29 @@ public:
     settings.setValue("customLabels", QVariant(customLabels.keys()));
   }
 
-  void saveCustomLabels() const {
-    QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
-    settings.beginGroup(QString::fromUtf8("TransferListFilters"));
-    settings.setValue("customLabels", QVariant(customLabels.keys()));
-  }
-
   void loadSettings() {
     QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
-    settings.beginGroup(QString::fromUtf8("TransferListFilters"));
-    statusFilters->setCurrentRow(settings.value("selectedFilterIndex", 0).toInt());
-    QStringList label_list = settings.value("customLabels", QStringList()).toStringList();
+    statusFilters->setCurrentRow(settings.value("TransferListFilters/selectedFilterIndex", 0).toInt());
+    const QStringList label_list = Preferences().getTorrentLabels();
     foreach(const QString &label, label_list) {
       customLabels.insert(label, 0);
       qDebug("Creating label QListWidgetItem: %s", qPrintable(label));
       QListWidgetItem *newLabel = new QListWidgetItem();
       newLabel->setText(label + " (0)");
-      newLabel->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder.png"));
+      newLabel->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("inode-directory"));
       labelFilters->addItem(newLabel);
     }
   }
 
 protected slots:
-  void updateTorrentNumbers(uint nb_downloading, uint nb_seeding, uint nb_active, uint nb_inactive, uint nb_paused) {
-    statusFilters->item(FILTER_ALL)->setData(Qt::DisplayRole, QVariant(tr("All")+" ("+QString::number(nb_active+nb_inactive)+")"));
-    statusFilters->item(FILTER_DOWNLOADING)->setData(Qt::DisplayRole, QVariant(tr("Downloading")+" ("+QString::number(nb_downloading)+")"));
-    statusFilters->item(FILTER_COMPLETED)->setData(Qt::DisplayRole, QVariant(tr("Completed")+" ("+QString::number(nb_seeding)+")"));
-    statusFilters->item(FILTER_PAUSED)->setData(Qt::DisplayRole, QVariant(tr("Paused")+" ("+QString::number(nb_paused)+")"));
-    statusFilters->item(FILTER_ACTIVE)->setData(Qt::DisplayRole, QVariant(tr("Active")+" ("+QString::number(nb_active)+")"));
-    statusFilters->item(FILTER_INACTIVE)->setData(Qt::DisplayRole, QVariant(tr("Inactive")+" ("+QString::number(nb_inactive)+")"));
+  void updateTorrentNumbers() {
+    const TorrentStatusReport report = transferList->getSourceModel()->getTorrentStatusReport();
+    statusFilters->item(FILTER_ALL)->setData(Qt::DisplayRole, QVariant(tr("All")+" ("+QString::number(report.nb_active+report.nb_inactive)+")"));
+    statusFilters->item(FILTER_DOWNLOADING)->setData(Qt::DisplayRole, QVariant(tr("Downloading")+" ("+QString::number(report.nb_downloading)+")"));
+    statusFilters->item(FILTER_COMPLETED)->setData(Qt::DisplayRole, QVariant(tr("Completed")+" ("+QString::number(report.nb_seeding)+")"));
+    statusFilters->item(FILTER_PAUSED)->setData(Qt::DisplayRole, QVariant(tr("Paused")+" ("+QString::number(report.nb_paused)+")"));
+    statusFilters->item(FILTER_ACTIVE)->setData(Qt::DisplayRole, QVariant(tr("Active")+" ("+QString::number(report.nb_active)+")"));
+    statusFilters->item(FILTER_INACTIVE)->setData(Qt::DisplayRole, QVariant(tr("Inactive")+" ("+QString::number(report.nb_inactive)+")"));
   }
 
   void torrentDropped(int row) {
@@ -302,22 +323,22 @@ protected slots:
     if(label.isEmpty() || customLabels.contains(label)) return;
     QListWidgetItem *newLabel = new QListWidgetItem();
     newLabel->setText(label + " (0)");
-    newLabel->setData(Qt::DecorationRole, QIcon(":/Icons/oxygen/folder.png"));
+    newLabel->setData(Qt::DecorationRole, IconProvider::instance()->getIcon("inode-directory"));
     labelFilters->addItem(newLabel);
     customLabels.insert(label, 0);
-    saveCustomLabels();
+    Preferences().addTorrentLabel(label);
   }
 
   void showLabelMenu(QPoint) {
     QMenu labelMenu(labelFilters);
     QAction *removeAct = 0;
     if(!labelFilters->selectedItems().empty() && labelFilters->row(labelFilters->selectedItems().first()) > 1)
-      removeAct = labelMenu.addAction(QIcon(":/Icons/oxygen/list-remove.png"), tr("Remove label"));
-    QAction *addAct = labelMenu.addAction(QIcon(":/Icons/oxygen/list-add.png"), tr("Add label..."));
+      removeAct = labelMenu.addAction(IconProvider::instance()->getIcon("list-remove"), tr("Remove label"));
+    QAction *addAct = labelMenu.addAction(IconProvider::instance()->getIcon("list-add"), tr("Add label..."));
     labelMenu.addSeparator();
-    QAction *startAct = labelMenu.addAction(QIcon(":/Icons/skin/play22.png"), tr("Resume torrents"));
-    QAction *pauseAct = labelMenu.addAction(QIcon(":/Icons/skin/pause22.png"), tr("Pause torrents"));
-    QAction *deleteTorrentsAct = labelMenu.addAction(QIcon(":/Icons/skin/delete22.png"), tr("Delete torrents"));
+    QAction *startAct = labelMenu.addAction(IconProvider::instance()->getIcon("media-playback-start"), tr("Resume torrents"));
+    QAction *pauseAct = labelMenu.addAction(IconProvider::instance()->getIcon("media-playback-pause"), tr("Pause torrents"));
+    QAction *deleteTorrentsAct = labelMenu.addAction(IconProvider::instance()->getIcon("edit-delete"), tr("Delete torrents"));
     QAction *act = 0;
     act = labelMenu.exec(QCursor::pos());
     if(act) {
@@ -372,7 +393,7 @@ protected slots:
     // Un display filter
     delete labelFilters->takeItem(row);
     // Save custom labels to remember it was deleted
-    saveCustomLabels();
+    Preferences().removeTorrentLabel(label);
   }
 
   void applyLabelFilter(int row) {
@@ -388,7 +409,8 @@ protected slots:
     }
   }
 
-  void torrentChangedLabel(QString old_label, QString new_label) {
+  void torrentChangedLabel(TorrentModelItem *torrentItem, QString old_label, QString new_label) {
+    Q_UNUSED(torrentItem);
     qDebug("Torrent label changed from %s to %s", qPrintable(old_label), qPrintable(new_label));
     if(!old_label.isEmpty()) {
       if(customLabels.contains(old_label)) {
@@ -415,10 +437,8 @@ protected slots:
     updateStickyLabelCounters();
   }
 
-  void torrentAdded(QModelIndex index) {
-    Q_ASSERT(index.isValid());
-    if(!index.isValid()) return;
-    const QString &label = transferList->getSourceModel()->index(index.row(), TR_LABEL).data(Qt::DisplayRole).toString().trimmed();
+  void handleNewTorrent(TorrentModelItem* torrentItem) {
+    QString label = torrentItem->data(TorrentModelItem::TR_LABEL).toString();
     qDebug("New torrent was added with label: %s", qPrintable(label));
     if(!label.isEmpty()) {
       if(!customLabels.contains(label)) {
@@ -442,10 +462,9 @@ protected slots:
     updateStickyLabelCounters();
   }
 
-  void torrentAboutToBeDeleted(QModelIndex index) {
-    Q_ASSERT(index.isValid());
-    if(!index.isValid()) return;
-    QString label = transferList->getSourceModel()->index(index.row(), TR_LABEL).data(Qt::DisplayRole).toString().trimmed();
+  void torrentAboutToBeDeleted(TorrentModelItem* torrentItem) {
+    Q_ASSERT(torrentItem);
+    QString label = torrentItem->data(TorrentModelItem::TR_LABEL).toString();
     if(!label.isEmpty()) {
       // Update label counter
       const int new_count = customLabels.value(label, 0) - 1;
